local ADDON, Addon = ...;

local FastSell = CreateFrame("Frame", ADDON .. "FastSellFrame", MerchantFrame);
Addon.FastSell = FastSell;

local BUTTON_SIZE = 20;

function FastSell:Initialize()
	FastSell:SetSize(128, 32);
	FastSell:ClearAllPoints();
	FastSell:SetPoint("BOTTOMLEFT", 4, -2);

	FastSell:PlaceSellGreyButton();
	FastSell:PlaceSellCrapButton();
end

--[[ core functions ]]--

Addon.SellItem = function(bag, slot)
	UseContainerItem(bag, slot);
end

local function IterateInventory(func)
	for bag=0,4 do
		for slot=0,GetContainerNumSlots(bag) do
		  local link = GetContainerItemLink(bag, slot);
		  func(link, bag, slot);
        end
    end
end

local function SellConditional(func)
	IterateInventory(function(link, bag, slot)
		if(not link) then return; end
		if(func(bag, slot, GetItemInfo(link))) then
			Addon.SellItem(bag, slot);
		end
	end);
end

local function IsSoulBound(bag, slot)
	GameTooltip:ClearLines();
	GameTooltip:SetOwner(WorldFrame, "ANCHOR_NONE");
	GameTooltip:SetHyperlink(GetContainerItemLink(bag, slot));

	for _, region in ipairs({GameTooltip:GetRegions()}) do
	   if region and region:GetObjectType() == "FontString" then
		  local text = region:GetText()
		  if text and strfind(text, ITEM_BIND_ON_PICKUP) then
			-- Item is in bag and bound on pickup - so its soulbound
			return true;
		  end
	   end
	end
end

local function StyleButton(btn, ico)
	local nt = btn:CreateTexture()
	nt:SetTexture([[Interface\Buttons\UI-Quickslot2]])
	nt:SetSize(BUTTON_SIZE, BUTTON_SIZE)
	nt:SetPoint('CENTER', 0, -1)
	btn:SetNormalTexture(nt)

	local pt = btn:CreateTexture()
	pt:SetTexture([[Interface\Buttons\UI-Quickslot-Depress]])
	pt:SetSize(BUTTON_SIZE, BUTTON_SIZE)
	pt:SetAllPoints(btn)
	btn:SetPushedTexture(pt)

	local ht = btn:CreateTexture()
	ht:SetTexture([[Interface\Buttons\ButtonHilight-Square]])
	ht:SetSize(BUTTON_SIZE, BUTTON_SIZE)
	ht:SetAllPoints(btn)
	btn:SetHighlightTexture(ht)
	
	local icon = btn:CreateTexture()
	icon:SetTexture(ico)
	icon:SetSize(BUTTON_SIZE, BUTTON_SIZE)
	icon:SetAllPoints(btn)
end

--[[ sell grey button ]]--

local function SellGreyButtonClick()
	SellConditional(function(bag, slot, ...)
		if select("#", ...) > 0 then
			return select(3, ...) == 0;
		end
		
		return false;
	end);
end

function FastSell:CreateSellGreyButton()
	local b = CreateFrame('Button', self:GetName() .. 'GreyButton', self);
	b:SetScript('OnClick', SellGreyButtonClick);
	self.SellGreyButton = b;
	return b;
end

function FastSell:GetSellGreyButton()
	return self.SellGreyButton;
end

function FastSell:PlaceSellGreyButton()
	local btn = self:GetSellGreyButton() or self:CreateSellGreyButton();

	btn:ClearAllPoints();
	btn:SetPoint("LEFT", 5, 1);
	btn:SetSize(BUTTON_SIZE, BUTTON_SIZE);
	
	StyleButton(btn, [[Interface\Icons\INV_Misc_Coin_04]]);

	btn:Show();
end

--[[ sell crap button ]]--

local function SellCrapButtonClick()
	SellConditional(function(bag, slot, ...)
		local link = select(2, ...);
		
		q = select(3, ...) == 0; -- Quality too low?
		
		s = GetContainerItemEquipmentSetInfo(bag, slot) == false; -- Is not in set
		e = IsEquippableItem(...); -- Can be equipped
		so = IsSoulBound(bag, slot); -- Is soulbound
		eq = not IsEquippedItem(link); -- Item is not equipped
		
		return q or (s and e and so and eq);
	end);
end

function FastSell:CreateSellCrapButton()
	local b = CreateFrame('Button', self:GetName() .. 'CrapButton', self, 'UIPanelCloseButton');
	b:SetScript('OnClick', SellCrapButtonClick);
	self.SellCrapButton = b;
	return b;
end

function FastSell:GetSellCrapButton()
	return self.SellGrapButton;
end

function FastSell:PlaceSellCrapButton()
	local btn = self:GetSellCrapButton() or self:CreateSellCrapButton();

	btn:ClearAllPoints();
	btn:SetPoint("LEFT", 25, 1);
	btn:SetSize(BUTTON_SIZE, BUTTON_SIZE);
	
	StyleButton(btn, [[Interface\Icons\INV_Misc_Coin_02]]);

	btn:Show();
end

FastSell:Initialize();
