
local ADDON, Addon = ...;

local slots = {};
local links = {};
local f = CreateFrame("Frame");
Addon.Slots = links;

Addon.T_LINK = 1
Addon.T_COUNT = 2
Addon.T_LOCKED = 3
Addon.T_QUALITY = 4
Addon.T_TEXTURE = 5
Addon.T_INSET = 6
Addon.T_TYPE = 7
Addon.T_TYPEINDEX = 8
Addon.T_SUBTYPE = 9
Addon.T_SUBTYPEINDEX = 10
Addon.T_EQUIPLOC = 11
Addon.T_SELLPRICE = 12
Addon.T_SOULBOUND = 13
Addon.T_ACCOUNTBOUND = 14
Addon.T_STATS = 15
Addon.T_BAG = 16
Addon.T_SLOT = 17
Addon.T_ID = 18
Addon.T_GS = 19

local EventHandler = { }
EventHandler.registered = { }

Addon.EventHandler = EventHandler

function EventHandler:RegisterEvent(class, event)
	if not class then return end
	if not self.registered[event] then self.registered[event] = {} end
	table.insert(self.registered[event], class);
end

function EventHandler:FireEvent(event, slot)
	if not self.registered[event] then return end
	for _, class in ipairs(self.registered[event]) do
		class[event](class, slot);
	end
end

local function ToIndex(bag, slot)
	return (bag < 0 and bag*100 - slot) or bag*100 + slot
end

function f:Load()
	self.firstVisit = true
	self.frame = CreateFrame('Frame')
	
	self.RegisterEvent = function(self, event)
		self.frame:RegisterEvent(event)
	end
	
	self.OnEvent = function(f, event, ...)
		if self[event] then
			self[event](self, event, ...)
		end		
	end

	self.frame:SetScript('OnEvent', self.OnEvent)
	self.frame:RegisterEvent('PLAYER_LOGIN')
end

function f:AddItem(bag, slot)
	self:UpdateItem(bag, slot);
end

function f:RemoveItem(bag, slot)
	local data = slots[ToIndex(bag, slot)]

	if data and next(data) then
		local prevLink = data[1]
		for i in pairs(data) do
			data[i] = nil
		end
	end
end

function f:UpdateItems(bag)
	for slot = 1, GetContainerNumSlots(bag) do
		self:UpdateItem(bag, slot)
	end
end

function f:UpdateItem(bag, slot)
	local linkData = {}
	local prevId = slots[ToIndex(bag, slot)]
	local texture, count, locked, quality, readable, lootable, link = GetContainerItemInfo(bag, slot)
	local id = nil
	
	if link then
		id = link .. bag*100 .. slot
	end
	
	----------

	if prevId and id ~= prevId then
		-- Remove old link
		links[prevId] = nil
		EventHandler:FireEvent("BAG_EVENT_REMOVE", prevId)
	end
	
	slots[ToIndex(bag, slot)] = id
	
	---------
	
	if not id then
		return
	end
	
	-- Add new link or update same
	links[id] = linkData
	
	local level, minlevel, ty, subType, stackCount, equipLoc, texture, sellPrice = select(4, GetItemInfo(link))
	local inSet = GetContainerItemEquipmentSetInfo(bag, slot)
	local soulbound, accountbound = Addon:TooltipInformationAvailable(bag, slot, ITEM_SOULBOUND, ITEM_ACCOUNTBOUND)
	local stats = GetItemStats(link)
	
	local tyIndex = IndexOf({ GetAuctionItemClasses() }, ty);
	local subTypeIndex = IndexOf({ GetAuctionItemSubClasses(2) }, subType);
	
	linkData[Addon.T_LINK] = link 
	linkData[Addon.T_COUNT] = count
	linkData[Addon.T_LOCKED] = locked
	linkData[Addon.T_QUALITY] = quality
	linkData[Addon.T_TEXTURE] = texture
	linkData[Addon.T_INSET] = inSet
	linkData[Addon.T_TYPE] = ty
	linkData[Addon.T_TYPEINDEX] = tyIndex
	linkData[Addon.T_SUBTYPE] = subType
	linkData[Addon.T_SUBTYPEINDEX] = subTypeIndex
	linkData[Addon.T_EQUIPLOC] = equipLoc
	linkData[Addon.T_SELLPRICE] = sellPrice
	linkData[Addon.T_SOULBOUND] = soulbound
	linkData[Addon.T_ACCOUNTBOUND] = accountbound
	linkData[Addon.T_STATS] = stats
	linkData[Addon.T_BAG] = bag
	linkData[Addon.T_SLOT] = slot
	linkData[Addon.T_ID] = id
	linkData[Addon.T_GS] = level
	
	if id == prevId then
		EventHandler:FireEvent("BAG_EVENT_UPDATE", linkData)
	else
		if EnabledItems[id] == nil then
			EnabledItems[id] = not Addon:IsCritical(linkData)
		end
		EventHandler:FireEvent("BAG_EVENT_ADD", linkData)
	end
	
	return id
end

function Addon:LinkToSlot(link)
	local level, minlevel, ty, subType, stackCount, equipLoc, texture, sellPrice = select(4, GetItemInfo(link))
	local stats = GetItemStats(link)
	
	local tyIndex = IndexOf({ GetAuctionItemClasses() }, ty);
	local subTypeIndex = IndexOf({ GetAuctionItemSubClasses(2) }, subType);
	
	local linkData = {};
	
	linkData[Addon.T_LINK] = link 
	linkData[Addon.T_COUNT] = count
	linkData[Addon.T_LOCKED] = locked
	linkData[Addon.T_QUALITY] = quality
	linkData[Addon.T_TEXTURE] = texture
	linkData[Addon.T_INSET] = nil
	linkData[Addon.T_TYPE] = ty
	linkData[Addon.T_TYPEINDEX] = tyIndex
	linkData[Addon.T_SUBTYPE] = subType
	linkData[Addon.T_SUBTYPEINDEX] = subTypeIndex
	linkData[Addon.T_EQUIPLOC] = equipLoc
	linkData[Addon.T_SELLPRICE] = sellPrice
	linkData[Addon.T_SOULBOUND] = nil
	linkData[Addon.T_ACCOUNTBOUND] = nil
	linkData[Addon.T_STATS] = stats
	linkData[Addon.T_BAG] = bag
	linkData[Addon.T_SLOT] = slot
	linkData[Addon.T_ID] = id
	linkData[Addon.T_GS] = level
	
	return linkData
end

function f:UpdateBagSize(bag)
	local prevSize = slots[bag*100] or 0
	local newSize = GetContainerNumSlots(bag) or 0
	slots[bag*100] = newSize

	if prevSize > newSize then
		for slot = newSize+1, prevSize do
			self:RemoveItem(bag, slot)
		end
	elseif prevSize < newSize then
		for slot = prevSize+1, newSize do
			self:AddItem(bag, slot)
		end
	end
end

function f:UpdateBagSizes()
	for bag = 1, NUM_BAG_SLOTS do
		self:UpdateBagSize(bag)
	end
end

function f:PLAYER_LOGIN()
	self:RegisterEvent('BAG_UPDATE')
	self:RegisterEvent('EQUIPMENT_SETS_CHANGED')

	self:UpdateBagSize(BACKPACK_CONTAINER)
	self:UpdateItems(BACKPACK_CONTAINER)
	
	for i = 0, 4 do
		if GetContainerNumSlots(i) > 0 then
			self:UpdateItems(i)
		end
	end
end

function f:BAG_UPDATE(event, bag)
	self:UpdateBagSizes()
	self:UpdateItems(bag)
end

function f:EQUIPMENT_SETS_CHANGED()
	for i = 0, 4 do
		if GetContainerNumSlots(i) > 0 then
			self:UpdateItems(i)
		end
	end
end

function Addon:TooltipInformationAvailable(bag, slot, ...)
   local hasMap = {};
   
   GameTooltip:ClearLines();
   GameTooltip:SetOwner(WorldFrame, "ANCHOR_NONE");
   GameTooltip:SetBagItem(bag, slot);
   
   for p, tag in ipairs({...}) do
		hasMap[p] = false
   end
   
   for _, region in ipairs({GameTooltip:GetRegions()}) do
      if region and region:GetObjectType() == "FontString" then
         local text = region:GetText()
		 
		 for p, tag in ipairs({...}) do
			 if text and strfind(text, tag) then
				table.insert(hasMap, p, true);
			 end
		 end
      end
   end
   
   GameTooltip:Hide();
   return unpack(hasMap);
end

function IndexOf(t, v)
	for i, s in ipairs(t) do
		if s == v then
			return i;
		end
	end
	return nil;
end

f:Load();