
local ADDON, Addon = ...;
local ItemSlot = CreateFrame('Button');

Addon.ItemSlot = ItemSlot;

ItemSlot.mt = { __index = ItemSlot };

local RIGHT_BUTTON_UP = "RightButtonUp"
local LEFT_BUTTON_UP = "LeftButtonUp"

function ItemSlot:New(slot, part)
	local b = setmetatable(self:CreateItemSlot(slot, part.Frame), self.mt);
	
	b.part = part;
	
	b:RegisterForClicks(LEFT_BUTTON_UP, RIGHT_BUTTON_UP);
	
	b:HookScript('OnClick', b.SellButtonClick);
	b:SetScript('OnEnter', b.SellButtonEnter);
	b:SetScript('OnLeave', b.SellButtonLeave);
	
	-- add a quality border texture
	local border = b:CreateTexture(nil, 'OVERLAY')
	border:SetSize(67, 67)
	border:SetPoint('CENTER', b)
	border:SetTexture([[Interface\Buttons\UI-ActionButton-Border]])
	border:SetBlendMode('ADD')
	border:Hide()
	
	b.Border = border;
	
	b:Set(slot);
	
	b:SetBorderColor();
	
	Addon.EventHandler:RegisterEvent(b, "BAG_EVENT_UPDATE");
	Addon.EventHandler:RegisterEvent(b, "BAG_EVENT_REMOVE");
	
	b:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
	
	b:SetScript('OnEvent', function(f, event, ...)
		if b[event] then
			b[event](b, event, ...);
		end
	end);
	
	return b;
end

function ItemSlot:PLAYER_EQUIPMENT_CHANGED()
	if Addon:IsWeapon(self.slot) or Addon:IsArmor(self.slot) then
		if Addon:IsCritical(self.slot) and self:GetEnabled() then
			self:Disable(true)
		elseif not self:GetEnabled() and not Addon:IsCritical(self.slot) then
			self:Enable(true)
		end
	end
end

function ItemSlot:BAG_EVENT_UPDATE(slot)
	if self.id ~= Addon:GetID(slot) then return end
	self:Set(slot);
end

function ItemSlot:BAG_EVENT_REMOVE(id)
	if self.id ~= id then return end
	self.part:RemoveItemBetween(id);
	EnabledItems[id] = nil
end

function ItemSlot:Remove()
	self.part:RemoveItemBetween(id);
	EnabledItems[id] = nil
end

function ItemSlot:CreateItemSlot(slot, parentFrame)
	local bag, slot = Addon:GetBagSlot(slot);
	return CreateFrame('Button', parentFrame:GetName() .. 'Button' .. 'Bag' .. bag .. 'Slot' .. slot, parentFrame, "ItemButtonTemplate");
end

function ItemSlot:Set(slot)
	self.id = Addon:GetID(slot);
	self.slot = slot;
	SetItemButtonTexture(self, Addon:GetItemTexture(slot));
	self:SetCount();
	self:UpdateState();
end

function ItemSlot:SetCount(slot)
	SetItemButtonCount(self, Addon:GetItemCount(self.slot))
end

function ItemSlot:SellButtonClick(button)
	if not self.part:GetEnabled() then return end
	if button == "RightButton" and not self.disabled then
		Addon:SellItem(self.slot);
	elseif button == "LeftButton" then
		if not self:GetEnabled() then
			self:Enable(true);
		else
			self:Disable(true);
		end
		self:SellButtonEnter();
	end
end

function ItemSlot:AnchorTooltip()
	if self:GetRight() >= (GetScreenWidth() / 2) then
		GameTooltip:SetOwner(self, 'ANCHOR_LEFT')
	else
		GameTooltip:SetOwner(self, 'ANCHOR_RIGHT')
	end
end
	
function ItemSlot:SellButtonEnter()
	ResetCursor();
	if self:GetEnabled() then
		SetCursor("BUY_CURSOR");
	end
	GameTooltip:ClearLines();
	self:AnchorTooltip();
	GameTooltip:SetBagItem(Addon:GetBagSlot(self.slot));
	GameTooltip:Show();
end
	
function ItemSlot:SellButtonLeave()
	GameTooltip:Hide();
	ResetCursor();
end
	
function ItemSlot:SetBorderColor()
	local quality = Addon:GetItemQuality(self.slot);
	
	if quality < 2 then 
		self.Border:Hide();
		return;
	end
	
	local r, g, b, a = GetItemQualityColor(Addon:GetItemQuality(self.slot));
	self.Border:SetVertexColor(r, g, b, a);
	self.Border:Show();
end

function ItemSlot:UpdateState()
	if not self.part:GetEnabled() then return end
	if self:GetEnabled() then
		self:Enable(true)
	else
		self:Disable(true)
	end
end

function ItemSlot:Disable(withSettings)
	self.disabled = true
	self:SetAlpha(0.4);
	self.Border:Hide();
	if withSettings then
		self:SetEnabled(false);
	end
end

function ItemSlot:Enable(withSettings)
	self.disabled = false
	self:SetAlpha(1);
	self:SetBorderColor();
	if withSettings then
		self:SetEnabled(true)
	end
end

function ItemSlot:SetEnabled(enabled)
	EnabledItems[Addon:GetID(self.slot)] = enabled;
end

function ItemSlot:GetEnabled()
	return EnabledItems[Addon:GetID(self.slot)];
end