local ADDON, Addon = ...;

local EventHandler = Addon.EventHandler;

local BUTTON_SIZE = 36;
local ITEMTYPE_WEAPON = 1
local ITEMTYPE_ARMOR = 2
local ITEMTYPE_USABLE = 4
local ITEMTYPE_WORKITEM = 6
local ITEMTYPE_OTHER = 9
local ITEMTYPE_GREY = 100

local PART_WEAPON = 1
local PART_ARMOR = 2
local PART_USABLE = 3
local PART_WORKITEM = 4
local PART_OTHER = 5
local PART_GREY = 6

local itemParts = {};
local itemFrame = CreateFrame("Frame", ADDON .. "ItemFrame", Addon.Vendor);

Addon.ItemFrame = itemFrame;

function itemFrame:Initialize()
	self:ClearAllPoints();
	self:SetPoint("TOPLEFT", 0, 0);
	self:SetSize(Addon.Vendor:GetSize());
	self:CreateParts();
	
	Addon.EventHandler:RegisterEvent(self, "BAG_EVENT_ADD");
	
end

function itemFrame:BAG_EVENT_ADD(slot)
	for _, part in pairs(itemParts) do
		part:AddItem(slot);
	end
end

function itemFrame:OrganizeParts()
	local prev = nil
	for i, part in ipairs(itemParts) do
		if part:NumShownItems() > 0 then
			part.Frame:ClearAllPoints();
			if not prev then
				part.Frame:SetPoint("TOPLEFT", MerchandrixItemFrame, "TOPLEFT", 10, -10);
			else
				part.Frame:SetPoint("TOP", "MerchandrixItemPart" .. i - 1 , "BOTTOM", 0, -5);
			end
			prev = part
		end
	end
end

function itemFrame:CreateParts()

	local function IsWeapon(slot)
		return Addon:AllowSellItem(slot) and Addon:IsWeapon(slot) and Addon:GetItemQuality(slot) ~= 0;
	end
	
	local function IsArmor(slot)
		return Addon:AllowSellItem(slot) and Addon:IsArmor(slot) and Addon:GetItemQuality(slot) ~= 0;
	end
	
	local function IsUsable(slot)
		local ty, tyIdx = Addon:GetItemTypes(slot);
		return Addon:AllowSellItem(slot) and tyIdx == ITEMTYPE_USABLE and Addon:CanSellItem(slot) and Addon:GetItemQuality(slot) ~= 0;
	end
	
	local function IsWorkItem(slot)
		local ty, tyIdx = Addon:GetItemTypes(slot);
		return Addon:AllowSellItem(slot) and tyIdx == ITEMTYPE_WORKITEM and Addon:CanSellItem(slot) and Addon:GetItemQuality(slot) ~= 0;
	end
	
	local function IsOther(slot)
		local ty, tyIdx = Addon:GetItemTypes(slot);
		return Addon:AllowSellItem(slot) and tyIdx == ITEMTYPE_OTHER and Addon:CanSellItem(slot) and Addon:GetItemQuality(slot) ~= 0;
	end
	
	local function IsGrey(slot)
		return Addon:GetItemQuality(slot) == 0 and Addon:CanSellItem(slot);
	end
	
	table.insert(itemParts, Addon.ItemPart:New(PART_WEAPON, nil, "Waffen", IsWeapon, true));
	table.insert(itemParts, Addon.ItemPart:New(PART_ARMOR, PART_WEAPON, "Rüstung", IsArmor, true));
	table.insert(itemParts, Addon.ItemPart:New(PART_USABLE, PART_ARMOR, "Verbrauchbar", IsUsable, false));
	table.insert(itemParts, Addon.ItemPart:New(PART_WORKITEM, PART_USABLE, "Handwerkswaren", IsWorkItem, false));
	table.insert(itemParts, Addon.ItemPart:New(PART_OTHER, PART_WORKITEM, "Verschiedenes", IsOther, false));
	table.insert(itemParts, Addon.ItemPart:New(PART_GREY, PART_OTHER, "Grau", IsGrey, true));
	
	self:UpdateSize();
end

function itemFrame:GetEnabledItems()
	local enabled = {}
	for _, part in pairs(itemParts) do
		for p, item in ipairs(part:GetEnabledItems()) do
			table.insert(enabled, item)
		end
	end
	
	return enabled
end

function itemFrame:UpdateSize()
	local height = 0;
	local width = 0;
	
	self:OrganizeParts();
	
	for _, part in ipairs(itemParts) do
		local w, h = part.Frame:GetSize();
		
		if w > width then
			width = w;
		end
		
		if part:NumShownItems() > 0 then
			height = height + h + 5;
		end
	end
	width = width + 20;
	
	self:SetSize(width, height);
	Addon.Vendor:UpdateSize();
end

itemFrame:Initialize();