
local ADDON, Addon = ...;

local ITEMTYPE_WEAPON = 1
local ITEMTYPE_ARMOR = 2
local ITEMTYPE_USABLE = 4
local ITEMTYPE_WORKITEM = 6
local ITEMTYPE_OTHER = 9
local ITEMTYPE_GREY = 100

local slots = {};
local links = {};
local new = {};
local old = {};
local newTypes = {};

Addon.TypeStore = { 
	["weapon"] = {},
	["armor"] = {},
	["usable"] = {},
	["workitem"] = {},
	["other"] = {},
	["grey"] = {}
};

local f = CreateFrame("Frame");
Addon.Slots = links;

Addon.T_LINK = 1
Addon.T_COUNT = 2
Addon.T_LOCKED = 3
Addon.T_QUALITY = 4
Addon.T_TEXTURE = 5
Addon.T_INSET = 6
Addon.T_TYPE = 7
Addon.T_TYPEINDEX = 8
Addon.T_SUBTYPE = 9
Addon.T_SUBTYPEINDEX = 10
Addon.T_EQUIPLOC = 11
Addon.T_SELLPRICE = 12
Addon.T_SOULBOUND = 13
Addon.T_ACCOUNTBOUND = 14
Addon.T_STATS = 15
Addon.T_BAG = 16
Addon.T_SLOT = 17
Addon.T_ID = 18
Addon.T_GS = 19
Addon.T_ADDED = 20
Addon.T_NAME = 21

local EventHandler = { }
EventHandler.registered = { }

Addon.EventHandler = EventHandler

function EventHandler:RegisterEvent(class, event)
	if not class then return end
	if not self.registered[event] then self.registered[event] = {} end
	table.insert(self.registered[event], class);
end

function EventHandler:FireEvent(event, slot)
	if not self.registered[event] then return end
	for _, class in ipairs(self.registered[event]) do
		class[event](class, slot);
	end
end

local function ToIndex(bag, slot)
	return (bag < 0 and bag*100 - slot) or bag*100 + slot
end

function f:Load()
	self.firstVisit = true
	self.frame = CreateFrame('Frame')
	
	self.RegisterEvent = function(self, event)
		self.frame:RegisterEvent(event)
	end
	
	self.OnEvent = function(f, event, ...)
		if self[event] then
			self[event](self, event, ...)
		end		
	end

	self.frame:SetScript('OnEvent', self.OnEvent)
	self.frame:RegisterEvent('PLAYER_LOGIN')
end

function f:UpdateItems(bag)
	if bag >= 0 and bag < 5 then
		for slot = 1, GetContainerNumSlots(bag) do
			self:UpdateItem(bag, slot)
		end
	end
end

local function GetItemTypeSlot(slot)
	local quality = Addon:GetItemQuality(slot);
	local ty, tyIdx = Addon:GetItemTypes(slot);
	local typeStore;
	
	if(quality == 0) then return "grey", Addon.TypeStore["grey"]; end
	
	local function get(where)
		return function() return where, Addon.TypeStore[where]; end;
	end
	
	if tyIdx == ITEMTYPE_WEAPON then
		return "weapon", Addon.TypeStore["weapon"];
	elseif tyIdx == ITEMTYPE_ARMOR then
		return "armor", Addon.TypeStore["armor"];
	elseif tyIdx == ITEMTYPE_USABLE then
		return "usable", Addon.TypeStore["usable"];
	elseif tyIdx == ITEMTYPE_WORKITEM then
		return "workitem", Addon.TypeStore["workitem"];
	elseif tyIdx == ITEMTYPE_OTHER then
		return "other", Addon.TypeStore["other"];
	end
end

local function AddItemToTypeStore(slot)
	local t, itemTypeSlot = GetItemTypeSlot(slot);
	if itemTypeSlot then
		itemTypeSlot[slot[Addon.T_ID]] = slot;
	end
end

function f:UpdateItem(bag, slot)
	local linkData = { };
	local prevId = slots[ToIndex(bag, slot)];
	local prev = links[prevId];
	local id = nil;
	local texture, count, locked, quality, readable, lootable, link = GetContainerItemInfo(bag, slot)
	
	if link then
		id = link .. ToIndex(bag, slot)
	end
	
	-- Track old item --
	if prev and prev ~= nil then
		prevId = prev[Addon.T_ID];
		
		if prevId then
			prev["Enabled"] = EnabledItems[prevId];
			if id ~= prevId and prevId then
				EnabledItems[prevId] = nil;
			end
			
			if id ~= prevId then
				table.insert(old, prev);
			end
			
			links[prevId] = nil;
			slots[ToIndex(bag, slot)] = nil;
			
			-- Remove old entry --
			local t, itemTypeSlot = GetItemTypeSlot(prev);
			if itemTypeSlot then
				itemTypeSlot[prevId] = nil;
			end
		end
	end
	
	if not id then return end;
	
	local name, lk, q, level, minlevel, ty, subType, stackCount, equipLoc, texture, sellPrice = GetItemInfo(link);
	
	if sellPrice then
		if sellPrice == 0 then
			return;
		end
		
		sellPrice = sellPrice * count
	end
	
	local inSet = GetContainerItemEquipmentSetInfo(bag, slot)
	
	if inSet then
		links[id] = nil;
		slots[ToIndex(bag, slot)] = nil;
		return;
	end
	
	--local soulbound, accountbound = Addon:TooltipInformationAvailable(bag, slot, ITEM_SOULBOUND, ITEM_ACCOUNTBOUND)
	--local stats = GetItemStats(link)
	
	local tyIndex = IndexOf({ GetAuctionItemClasses() }, ty);
	local subTypeIndex = nil;
	if tyIndex then
		subTypeIndex = IndexOf({ GetAuctionItemSubClasses(tyIndex) }, subType);
	end
	
	linkData[Addon.T_LINK] = link 
	linkData[Addon.T_NAME] = name
	linkData[Addon.T_COUNT] = count
	linkData[Addon.T_LOCKED] = locked
	linkData[Addon.T_QUALITY] = quality
	linkData[Addon.T_TEXTURE] = texture
	linkData[Addon.T_INSET] = inSet
	linkData[Addon.T_TYPE] = ty
	linkData[Addon.T_TYPEINDEX] = tyIndex
	linkData[Addon.T_SUBTYPE] = subType
	linkData[Addon.T_SUBTYPEINDEX] = subTypeIndex
	linkData[Addon.T_EQUIPLOC] = equipLoc
	linkData[Addon.T_SELLPRICE] = sellPrice
	--linkData[Addon.T_SOULBOUND] = soulbound
	--linkData[Addon.T_ACCOUNTBOUND] = accountbound
	--linkData[Addon.T_STATS] = stats
	linkData[Addon.T_BAG] = bag
	linkData[Addon.T_SLOT] = slot
	linkData[Addon.T_ID] = id
	linkData[Addon.T_GS] = level

	if EnabledItems[id] == nil then
		EnabledItems[id] = true;
	end
	
	if prevId ~= id then
		table.insert(new, linkData);
	end;
	
	slots[ToIndex(bag, slot)] = id;
	links[id] = linkData;
	
	local t, typeSlot = GetItemTypeSlot(linkData);
	if typeSlot then
		typeSlot[id] = linkData;
		newTypes[id] = t;
	end
	return id
end

function Addon:LinkToSlot(link)
	local name, lk, q, level, minlevel, ty, subType, stackCount, equipLoc, texture, sellPrice = GetItemInfo(link)
	--local stats = GetItemStats(link)
	
	local tyIndex = IndexOf({ GetAuctionItemClasses() }, ty);
	local subTypeIndex = nil;
	if tyIndex then
		subTypeIndex = IndexOf({ GetAuctionItemSubClasses(tyIndex) }, subType);
	end
	
	local linkData = {};
	
	linkData[Addon.T_LINK] = link 
	linkData[Addon.T_NAME] = name
	linkData[Addon.T_COUNT] = count
	linkData[Addon.T_LOCKED] = locked
	linkData[Addon.T_QUALITY] = quality
	linkData[Addon.T_TEXTURE] = texture
	linkData[Addon.T_INSET] = nil
	linkData[Addon.T_TYPE] = ty
	linkData[Addon.T_TYPEINDEX] = tyIndex
	linkData[Addon.T_SUBTYPE] = subType
	linkData[Addon.T_SUBTYPEINDEX] = subTypeIndex
	linkData[Addon.T_EQUIPLOC] = equipLoc
	linkData[Addon.T_SELLPRICE] = sellPrice
	linkData[Addon.T_SOULBOUND] = nil
	linkData[Addon.T_ACCOUNTBOUND] = nil
	--linkData[Addon.T_STATS] = stats
	linkData[Addon.T_BAG] = bag
	linkData[Addon.T_SLOT] = slot
	linkData[Addon.T_ID] = id
	linkData[Addon.T_GS] = level
	
	return linkData
end

function f:PLAYER_LOGIN()
	self:RegisterEvent('BAG_UPDATE')
	self:RegisterEvent('BAG_UPDATE_DELAYED')
	self:RegisterEvent('EQUIPMENT_SETS_CHANGED')
	
	for i = 0, 4 do
		if GetContainerNumSlots(i) > 0 then
			self:UpdateItems(i)
		end
	end
end

function f:BAG_UPDATE(event, bag)
	self:UpdateItems(bag)
end

function f:BAG_UPDATE_DELAYED(event, ...)
	for _, o in ipairs(old) do
		for _, n in ipairs(new) do
			if o[Addon.T_LINK] == n[Addon.T_LINK] then
				local id = n[Addon.T_ID]
				local enabled = o["Enabled"]
				if enabled ~= nil then
					EnabledItems[id] = enabled;
				end
			end
		end
	end

	EventHandler:FireEvent("BAG_EVENT_UPDATE");
	
	old = {}
	new = {}
end

function f:EQUIPMENT_SETS_CHANGED()
	for i = 0, 4 do
		if GetContainerNumSlots(i) > 0 then
			self:UpdateItems(i)
		end
	end
	
	for _, v in pairs(newTypes) do 
		EventHandler:FireEvent("BAG_EVENT_UPDATE", v);
	end
	
	old = {}
	new = {}
	newTypes = {}
end

function Addon:TooltipInformationAvailable(bag, slot, ...)
   local hasMap = {};
   
   GameTooltip:ClearLines();
   GameTooltip:SetOwner(WorldFrame, "ANCHOR_NONE");
   GameTooltip:SetBagItem(bag, slot);
   
   for p, tag in ipairs({...}) do
		hasMap[p] = false
   end
   
   for _, region in ipairs({GameTooltip:GetRegions()}) do
      if region and region:GetObjectType() == "FontString" then
         local text = region:GetText()
		 
		 for p, tag in ipairs({...}) do
			 if text and strfind(text, tag) then
				table.insert(hasMap, p, true);
			 end
		 end
      end
   end
   
   GameTooltip:Hide();
   return unpack(hasMap);
end

function IndexOf(t, v)
	for i, s in ipairs(t) do
		if s == v then
			return i;
		end
	end
	return nil;
end

f:Load();