
local ADDON, Addon = ...;
local ItemSlot = CreateFrame('Button');

Addon.ItemSlot = ItemSlot;

ItemSlot.mt = { __index = ItemSlot };

local RIGHT_BUTTON_UP = "RightButtonUp"
local LEFT_BUTTON_UP = "LeftButtonUp"

function ItemSlot:New(slot, part)
	local b = setmetatable(self:CreateItemSlot(slot, part.Frame), self.mt);
	
	b.part = part;
	
	b:RegisterForClicks(LEFT_BUTTON_UP, RIGHT_BUTTON_UP);
	
	b:HookScript('OnClick', b.SellButtonClick);
	b:SetScript('OnEnter', b.SellButtonEnter);
	b:SetScript('OnLeave', b.SellButtonLeave);
	
	-- add a quality border texture
	local border = b:CreateTexture(nil, 'OVERLAY')
	border:SetPoint('CENTER', b)
	border:SetTexture([[Interface\Buttons\UI-ActionButton-Border]])
	border:SetBlendMode('ADD')
	border:Hide()
	
	b.Border = border;
	
	b:Set(slot);
	
	b:UseConfig();
	
	b:UpdateState();
	
	b:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
	b:RegisterEvent("MODIFIER_STATE_CHANGED")
	
	b:SetScript('OnEvent', function(f, event, ...)
		if b[event] then
			b[event](b, event, ...);
		end
	end);
	
	return b;
end

function ItemSlot:UseConfig()
	local size = MerchandrixConfig.general.buttonSize
	local multi = 64 / 39
	
	self:SetSize(size, size)
	self.Border:SetSize(size * multi, size * multi)
	Addon:StyleButton(self, nil, size)
end

function ItemSlot:MODIFIER_STATE_CHANGED(event, button, state)
	if button ~= "LSHIFT" and button ~= "RSHIFT" then return end
	
	if state == 1 then
		GameTooltip_ShowCompareItem()
	else
		if ShoppingTooltip1 then
			ShoppingTooltip1:Hide()
		end
		if ShoppingTooltip2 then
			ShoppingTooltip2:Hide()
		end
		if ShoppingTooltip3 then
			ShoppingTooltip3:Hide()
		end
	end
end

function ItemSlot:PLAYER_EQUIPMENT_CHANGED()
	--if Addon:IsWeapon(self.slot) or Addon:IsArmor(self.slot) then
	--	if self:GetEnabled() then
	--		self:Disable(true)
	--	elseif not self:GetEnabled() and not Addon:IsCritical(self.slot) then
	--		self:Enable(true)
	--	end
	--end
end

function ItemSlot:Remove()
	if not self:IsShown() then return end
	self.part:RemoveItemBetween(self.id);
end

function ItemSlot:CreateItemSlot(slot, parentFrame)
	local bag, slot = Addon:GetBagSlot(slot);
	return CreateFrame('Button', parentFrame:GetName() .. 'Button' .. 'Bag' .. bag .. 'Slot' .. slot, parentFrame, "ItemButtonTemplate");
end

function ItemSlot:Set(slot)
	self.id = Addon:GetID(slot);
	self.slot = slot;
	SetItemButtonTexture(self, Addon:GetItemTexture(slot));
	self:SetCount();
	self:UpdateState();
end

function ItemSlot:SetCount(slot)
	SetItemButtonCount(self, Addon:GetItemCount(self.slot))
end

function ItemSlot:SellButtonClick(button)
	if not self.part:GetEnabled() then return end
	if button == "RightButton" and self:GetEnabled() then
		Addon:SellItem(self.slot);
	elseif button == "LeftButton" then
		self:SetEnabled(not self:GetEnabled());
		self:SellButtonEnter();
	end
end

function ItemSlot:AnchorTooltip()
	if self:GetRight() >= (GetScreenWidth() / 2) then
		GameTooltip:SetOwner(self, 'ANCHOR_LEFT')
	else
		GameTooltip:SetOwner(self, 'ANCHOR_RIGHT')
	end
end
	
function ItemSlot:SellButtonEnter()
	ResetCursor();
	if self:GetEnabled() then
		SetCursor("BUY_CURSOR");
	end
	GameTooltip:ClearLines();
	self:AnchorTooltip();
	GameTooltip:SetBagItem(Addon:GetBagSlot(self.slot));
	GameTooltip:Show();
end
	
function ItemSlot:SellButtonLeave()
	GameTooltip:Hide();
	ResetCursor();
end
	
function ItemSlot:SetBorderColor()
	local quality = Addon:GetItemQuality(self.slot);
	
	if quality < 2 or not self:GetEnabled() then 
		self.Border:Hide();
		return;
	end
	
	local r, g, b, a = GetItemQualityColor(Addon:GetItemQuality(self.slot));
	self.Border:SetVertexColor(r, g, b, a);
	self.Border:Show();
end

function ItemSlot:UpdateState()
	self:SetEnabled(self:GetEnabled());
end

function ItemSlot:SetEnabled(enabled)
	local id = Addon:GetID(self.slot);
	if enabled ~= nil then
		if self.part:GetEnabled() then
			EnabledItems[id] = enabled;
		end
		
		if enabled then
			self:SetAlpha(1);
		else
			self:SetAlpha(0.3);
		end
	end
	self:SetBorderColor();
end

function ItemSlot:GetEnabled()
	return not not EnabledItems[Addon:GetID(self.slot)] and self.part:GetEnabled();
end