local ADDON, Addon = ...;

local EventHandler = Addon.EventHandler;

local BUTTON_SIZE = 36;
local ITEMTYPE_WEAPON = 1
local ITEMTYPE_ARMOR = 2
local ITEMTYPE_USABLE = 4
local ITEMTYPE_WORKITEM = 6
local ITEMTYPE_OTHER = 9
local ITEMTYPE_GREY = 100

local PART_WEAPON = 1
local PART_ARMOR = 2
local PART_USABLE = 3
local PART_WORKITEM = 4
local PART_OTHER = 5
local PART_GREY = 6

local itemParts = {};
local itemFrame = CreateFrame("Frame", ADDON .. "ItemFrame", Addon.Vendor);

Addon.ItemFrame = itemFrame;

function itemFrame:Initialize()
	self:ClearAllPoints();
	self:SetPoint("TOPLEFT", 0, 0);
	self:SetSize(Addon.Vendor:GetSize());
	self:CreateParts();
	
	self:RegisterEvent("ADDON_LOADED");
	
	self:SetScript("OnEvent", function(f, event, ...)
		if self[event] then
			self[event](self, event, ...)
		end		
	end);
end

function itemFrame:ADDON_LOADED()
	self:UseConfig()
end

function itemFrame:OrganizeParts()
	local prev = nil
	for i, part in ipairs(itemParts) do
		if part:NumShownItems() > 0 then
			part.Frame:ClearAllPoints();
			if not prev then
				part.Frame:SetPoint("TOPLEFT", MerchandrixItemFrame, "TOPLEFT", 10, -10);
			else
				part.Frame:SetPoint("TOP", prev.Frame , "BOTTOM", 0, -5);
			end
			prev = part
		end
	end
end

function itemFrame:CreateParts()
	table.insert(itemParts, Addon.ItemPart:New(PART_WEAPON, nil, Addon.L["Waffen"], "weapon", true));
	table.insert(itemParts, Addon.ItemPart:New(PART_ARMOR, PART_WEAPON, Addon.L["Rüstung"], "armor", true));
	table.insert(itemParts, Addon.ItemPart:New(PART_USABLE, PART_ARMOR, Addon.L["Verbrauchbar"], "usable", false));
	table.insert(itemParts, Addon.ItemPart:New(PART_WORKITEM, PART_USABLE, Addon.L["Handwerkswaren"], "workitem", false));
	table.insert(itemParts, Addon.ItemPart:New(PART_OTHER, PART_WORKITEM, Addon.L["Verschiedenes"], "other", false));
	table.insert(itemParts, Addon.ItemPart:New(PART_GREY, PART_OTHER, Addon.L["Grau"], "grey", true));
	
	self:UpdateSize();
end

function itemFrame:UseConfig()
	for _, part in ipairs(itemParts) do
		part:UseConfig()
	end
end

function itemFrame:GetEnabledItems()
	local enabled = {}
	for _, part in ipairs(itemParts) do
		for p, item in ipairs(part:GetEnabledItems()) do
			table.insert(enabled, item)
		end
	end
	
	return enabled
end

function itemFrame:UpdateSize()
	local height = 0;
	local width = 0;
	
	self:OrganizeParts();
	
	for _, part in ipairs(itemParts) do
		local w, h = part.Frame:GetSize();
		
		if w > width then
			width = w;
		end
		
		if part:NumShownItems() > 0 then
			height = height + h + 5;
		end
	end
	width = width + 20;
	
	self:SetSize(width, height);
	Addon.Vendor:UpdateSize();
end

itemFrame:Initialize();